# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2


class image_descriptor:
    def __init__(self,stream):
        self.stream=stream
        self.internal_position=self.stream.pos


    def read(self):
        self.internal_position=self.stream.pos
        self.Separator = self.stream.byte(value=0x2C)             # Image Descriptor identifier
        self.Left      = self.stream.word()                       # X position of image on the display
        self.Top       = self.stream.word()                       # Y position of image on the display
        self.Width     = self.stream.word()                       # Width of the image in pixels
        self.Height    = self.stream.word()                       # Height of the image in pixels
        self.Packed    = self.stream.byte()                       # Image and Color Table Data Information

        # computed
        self.unpack()

    def write(self):
        self.internal_position=self.stream.pos
        self.pack()
        self.stream.write_byte(self.Separator)
        self.stream.write_word(self.Left     )
        self.stream.write_word(self.Top      )
        self.stream.write_word(self.Width    )
        self.stream.write_word(self.Height   )
        self.stream.write_byte(self.Packed   )

    def new(self,   Separator=0x2C,
                    Left=0,
                    Top=0,
                    Width=300,
                    Height=240,
                    LocalColorTableFlag=0,
                    InterlaceFlag=0,
                    SortFlag=0,
                    Reserved=0,
                    LocalColorTableSize=0):
        self.Separator=Separator
        self.Left     =Left
        self.Top      =Top
        self.Width    =Width
        self.Height   =Height
        # computed
        self.LocalColorTableFlag  =LocalColorTableFlag
        self.InterlaceFlag        =InterlaceFlag
        self.SortFlag             =SortFlag
        self.Reserved             =Reserved
        self.LocalColorTableSize  =LocalColorTableSize

        self.pack()

    def unpack(self):
        self.LocalColorTableFlag  =self.stream.bit(self.Packed,7)
        self.InterlaceFlag        =self.stream.bit(self.Packed,6)
        self.SortFlag             =self.stream.bit(self.Packed,5)
        self.Reserved             =self.stream.bit(self.Packed,3,2)
        self.LocalColorTableSize  =self.stream.bit(self.Packed,0,3)
        self.ColorTableLength     =1 << (self.LocalColorTableSize + 1)

    def pack(self):
        self.Packed=0
        self.Packed+=self.LocalColorTableFlag  <<7
        self.Packed+=self.InterlaceFlag        <<6
        self.Packed+=self.SortFlag             <<5
        self.Packed+=self.Reserved             <<3
        self.Packed+=self.LocalColorTableSize
        

        #Group 1 : Every 8th. row, starting with row 0.              (Pass 1)
        #Group 2 : Every 8th. row, starting with row 4.              (Pass 2)
        #Group 3 : Every 4th. row, starting with row 2.              (Pass 3)
        #Group 4 : Every 2nd. row, starting with row 1.              (Pass 4)
    
    def debug(self):
        print("Image Descriptor Block")
        print("  Offset:              {0:02X}".format(self.internal_position))
        print("  Separator:           {0:02X}".format(self.Separator))
        print("  Left:                {0:02X}".format(self.Left))
        print("  Top:                 {0:02X}".format(self.Top))
        print("  Width:               {0:02X}".format(self.Width))
        print("  Height:              {0:02X}".format(self.Height))
        print("  Packed:              {0:02X}".format(self.Packed))
        print("  LocalColorTableFlag: {0}".format(self.LocalColorTableFlag))
        print("  InterlaceFlag:       {0}".format(self.InterlaceFlag))
        print("  SortFlag:            {0}".format(self.SortFlag))
        print("  Reserved:            {0:02X}".format(self.Reserved))
        print("  LocalColorTableSize: {0:02X}".format(self.LocalColorTableSize))



# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2

    

from .decode import decode
from .canvas import canvas

class gif:
    def __init__(self,debug=None):
        self.debug=debug

    def decode(self,filename):
        d=decode(filename,debug=self.debug)
        data=d.get()
        return data

    def canvas_it(self,gif_filename,html_filename):
        d=decode(gif_filename,debug=self.debug)
        data=d.get()
        canvas().web(html_filename,data)
        
    def extract(self,gif_filename,output_filename):
        d=decode(gif_filename,debug=self.debug)
        data=d.get()
        canvas().extract(data,output_filename)
    
    def screen(self,data,output_filename):
        canvas().screen_canvas(data,output_filename)

    








def match_color_index(r,g,b,color_table):
    last_distance=-1
    mappeded_color=-1

    color_table_len=len(color_table)
    for i in range(0,color_table_len):
        color=color_table[i]
        color_distance=(r-color[0])*(r-color[0])+(g-color[1])*(g-color[1])+(b-color[2])*(b-color[2])
        if last_distance==-1 or color_distance<last_distance:
            last_distance=color_distance
            mappeded_color=i

    return mappeded_color

# todo account for color table size mismatch, crud on new table, and reindexing for best color palette...
def remap(src_color_table,src_pixels,dst_color_table,dst_pixels):
    hash_map=[0]*len(src_color_table)
    # remap the colors from the source to the dest
    for i in src_color_table:
        src_color=src_color_table[i]
        new_index=match_color_index(src_color[0],src_color[1],src_color[2])
        hash_map[i]=new_index

    # reindex the pixels
    src_pixel_len=len(src_pixels)
    for i in range(0,src_pixel_len):
        original_index=src_pixels[i]
        #replace srrc data pixel...
        src_pixels[i]=has_map[original_index]





# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2


from .stream import DataStream
from .header import gif_header
from .image_descriptor import image_descriptor
from .image import image
from .graphics_control_extension import graphics_control_extension
from .application_extension import application_extension
from .CommentExtension import CommentExtension
from .PlainTextExtension import PlainTextExtension
from .trailer import trailer
from .color_table import gif_color_table



class encode_gif:
    def __init__(self,loop_count=0xFFFF,debug=None,auto=True):
        self.stream  =None
        self.header =None
        self.global_color_table=None
        self.loop_count=loop_count
        self.frames  =[]
        self.debug   =debug
        self.auto    =auto
  
    def write(self):
        if self.auto:
          return
        # auto computes packed values on write
        self.header.write()
        
        # write GTC if available
        if self.global_color_table:
          self.global_color_table.write()

          self.application_extension.write()
        
        #process frames
        for frame in self.frames:
            if frame['gce']: 
              frame['gce'].write()
              if self.debug: frame['gce'].debug()

            if frame['descriptor']:
              frame['descriptor'].write()
              if self.debug: frame['descriptor'].debug()

            if frame['color_table']:
              frame['color_table'].write()
              if self.debug: frame['color_table'].debug()
            
            if frame['image']:
              frame['image'].write()
              if self.debug: frame['image'].debug()

        # write terminator
        self.trailer.write()
        self.stream.close()

    def close(self):
      if self.auto:
        self.trailer.write()
        self.stream.close()

  
    def create(self,filename,
            width,
            height,
            default_palette=True,
            palette=None):

        self.stream=DataStream(filename,mode='w')
        
        self.add_header(width=width,height=height,palette=palette,default_palette=default_palette)
        self.application_extension=application_extension(self.stream)
        self.application_extension.new_netscape_block(loop_count=self.loop_count)
        
        if self.auto:
          self.application_extension.write()
          if self.debug: self.application_extension.debug()
        
        self.frames=[]
        
        self.trailer=trailer(self.stream)
        self.trailer.new()

        # create the header
    # Step 1, create a header
    # auto adds a default palette or given palette
    def add_header(self,width=320,height=240,palette=None,default_palette=True):
        self.header=gif_header(self.stream)
        self.header.new()
        self.header.ScreenWidth  = width
        self.header.ScreenHeight = height
        self.global_color_table=None

        if default_palette or palette:
          res=self.add_ct(palette=palette,default_palette=default_palette)
          self.global_color_table             =res['table']
          self.header.GlobalColorTableSize    =res['size']
          self.header.GlobalColorTableFlag    =res['flag']
          self.header.pack()
        if self.debug:
          self.header.debug()
        if self.auto:
          self.header.write()
          if default_palette or palette:
            self.global_color_table.write()
            if self.debug: self.global_color_table.debug()


        
    # STEP 2 adding a global palette to the gif 
    def add_ct(self,palette,default_palette=True):
        if palette or default_palette:
            color_table =gif_color_table(self.stream)
            color_table.new(palette=palette)
            color_table_size=color_table.get_byte_size()
            if color_table_size==0:
              color_table_flag=0
            else:
              color_table_flag=1
        else:
          color_table=None
          color_table_flag=0
          color_table_size=0

        return {'table':color_table,
                'flag':color_table_flag,
                'size':color_table_size }
        
    
    
    def add_frame(self,disposal_method=0,delay=1, transparent=None,top=0,left=0,width=None,height=None,palette=None,image_data=None):
        if None==width:
            width=self.header.ScreenWidth
        if None==height:
            height=self.header.ScreenHeight


        gce=graphics_control_extension(self.stream)
        gce.new(    DelayTime=delay,
                    ColorIndex=transparent,
                    DisposalMethod=disposal_method)
        descriptor=image_descriptor(self.stream)
        

        if palette:
            res=self.add_ct(palette)
            local_color_table    = res['table']
            LocalColorTableSize  = res['size']
            LocalColorTableFlag  = res['flag']
        else:
            local_color_table= None
            LocalColorTableFlag =0
            LocalColorTableSize =0
        
        descriptor.new(Left=left,Top=top,Width=width,Height=height,LocalColorTableFlag=LocalColorTableFlag,LocalColorTableSize=LocalColorTableSize)

        
        imagedata=image(self.stream)
        imagedata.new(data=image_data,min_code_size=self.header.GlobalColorTableSize+1)

        if self.auto:
            if gce:
              gce.write()
              if self.debug: gce.debug()
            if descriptor:
              descriptor.write()
              if self.debug:  descriptor.debug()
            if local_color_table:
              local_color_table.write()
              if self.debug: local_color_table.debug()
            if imagedata:
              imagedata.write()
              if self.debug: imagedata.debug()



        self.frames.append({'gce':gce,'descriptor':descriptor,'color_table':local_color_table,'image':imagedata})






# cython: profile=True
# cython: linetrace=True
# cython: binding=True

import sys
import time
import select


def has_stdin():
    if select.select([sys.stdin,],[],[],0.0)[0]:
        if sys.stdin.isatty()==True:
            return None
        else:
            return True
    return None

def read_stdin():
    ts = round(time.time(),8)
    events=[]
    try:
        o="start"
        sys.stdin.flush()
        while o:
            #if sys.stdin.closed:

            o= sys.stdin.readline()
            #print o
            timestamp=round(time.time(),8)-ts
            #print timestamp
            events.append([timestamp,'o',o])
        #print len(events)
    except KeyboardInterrupt:

        sys.stdout.flush()
        pass

    stream={'version':2,
        'width':80,
        'height':25,
        'timestamp':ts,
        'title':'',
        'env':'',
        'events':events}
    

    return stream



# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2


class trailer:
    def __init__(self,stream):
        self.stream=stream
        self.Trailer=None

    def read(self):
        self.internal_position=self.stream.pos
        self.Trailer=self.stream.byte(value=0x3B)
    
    def write(self):
        self.stream.write_byte(0x3B)

    def new(self):
        self.Trailer=0x3B

    def debug(self):
        print("Trailer")
        print("  Offset: {0:02X}".format(self.internal_position))





# -*- coding: utf-8 -*-
import sys

name=sys.argv[1]
o=[]
bracketed_paste_end="\033[?2004l"
bracketed_paste_start="\033[?2004h"
start_nocode=u"\033[200~"
end_nocode=u"\033[201~"
#print bracketed_paste_start.encode('latin-1'),

print ""
print(bracketed_paste_start+" Font: {0}\r".format(name))


o=[" {0:03} ".format(0)]
for i in range(0,256):
    o.append(unichr(i))
    if len(o)==64+1 or i==255:
        o.append(" {0:03}".format(i))
        print start_nocode.encode('latin-1')+ "".join(o).encode('latin-1') + end_nocode.encode('latin-1'),
        print u"\n\r", 
        o=[" {0:03} ".format(i+1)]

print bracketed_paste_end +"\r",


# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2

import os
import argparse
from .asciicast.reader import asciicast_reader
from .cast2gif import cast2gif
from .version import __version__
from .tools.passthrough import has_stdin, read_stdin
import pprint
import time
import sys



def main():
    print("ttygif version [{0}]\n".format( __version__))
    print(" - Compiled against Python: {0}".format(sys.version.replace("\n"," ")))

    parser = argparse.ArgumentParser(
        prog='ttygif',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description="""tty output to gif""")
    # actions

    parser.add_argument('--input',  '-i',   help='asciinema .cast file', default= None,metavar='FILE')
    parser.add_argument('--output', '-o',   help='gif output file. will default to ttygif-xx', default= None,metavar='FILE')
    parser.add_argument('--loop',   '-l',   help='number of loops to play, 0=unlimited', default=0,metavar='COUNT')
    parser.add_argument('--delay',  '-d',   help='delay before restarting gif in milliseconds ', default=100,metavar='MS')
    parser.add_argument('--record',         help='output generated cast data to file', metavar='FILE')
    parser.add_argument('--font',   '-f',   help='which internal font to use', metavar='NAME')
    
    parser.add_argument('--theme'   ,  '-t',   help='load custom theme: game,windows7,mac,fwdm,opensource,scripted,bar',default='default')
    parser.add_argument('--dilation',          help='process events at a faster or slower rate of time', default=1,metavar='RATE', type=float)
    parser.add_argument('--fps'     ,          help='encode at (n) frames per second (0-25) 0=speed of cast file, min 3ms', default=0,metavar='FPS', type=int)
    parser.add_argument('--columns' ,  '-c',  help='change character width of gif, default is 80 or what is in the cast file',metavar='WIDTH', type=int)
    parser.add_argument('--rows'    ,  '-r',  help='change character height of gif, default is 25 or what is in the cast file',metavar='HEIGHT', type=int)
    #parser.add_argument('--text-at'  help='print the text screen buffer at TIME',metavar='TIME', type=int)
 
    # underlay_display =simple, stretch, center
    # bounds x,y x x2,y2
    # underlay_display =simple, stretch, center
    # bounds x,y x x2,y2
    
    
    parser.add_argument('--debug',   help='show debuging statistics', action='store_true',default=None)
    #debug.add_argument('-c', '--show-commands', help='dump interpreted cast data ', action='store_true')
    
    # dev options
    #tools = parser.add_argument_group('Tools')
    #tools.add_argument('-x', '--extract',       help='gif data to json', action='store_true')
    #tools.add_argument('-w', '--html',          help='gif to html', action='store_true')

    args = parser.parse_args()
    if args.output==None:
        for index in range(0,10000):
            filename="ttygif-{0:04d}.gif".format(index)
            if os.path.exists(filename)==False:
                args.output=filename
                break;
        if args.output==None:
            print( " Well, Well. You have no output file, and all the auto generated file names are taken. Messy.")
            exit(1);

    


    #if args.html:
    #    gif().canvas_it(args.input,args.output)
    
    #elif args.extract:
    #    gif(debug=None).extract(args.input,args.output)

    #elif  args.show_commands and args.input:
    #    cast=asciicast_reader(debug=args.debug)
    #    stream=cast.load(args.input)
    #    v=viewer()
    #    for event in stream['events']:
    #        v.add_event(event)
#
    #    v.debug_sequence()

    
    
    if  args.output:
        events=None
        if has_stdin():
            events=read_stdin()
            if args.record:
                ar=asciicast_reader()
                ar.write(args.record,events)
            
        elif None==args.input:
            parser.print_help()    
            events=None
            exit(0)

        frame_rate=args.fps
        if frame_rate<0:
            frame_rate=1
        if frame_rate>100:
           frame_rate=100
            
        natural=None
        if args.fps==0:
            natural=True
        debug=args.debug
        try:
            
            cast2gif(args.input,args.output,
                    events=events,
                    loop_count=args.loop,
                    loop_delay=args.delay,
                    debug=debug,
                    dilation=args.dilation,
                    frame_rate=frame_rate,
                    natural=natural,
                    height=args.rows,
                    width=args.columns,
                    underlay=None,
                    #overlay=args.overlay,
                    font_name=args.font,
                    theme_name=args.theme)
        except KeyboardInterrupt:
            print("\nProcessing Aborted...")
            sys.exit()
    else:
        parser.print_help()
                    

if __name__=='__main__':
    main()



# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2


class graphics_control_extension:
    def __init__(self,stream):
        self.stream=stream
        self.internal_position=self.stream.pos

    def read(self):
        self.internal_position  = self.stream.pos
        self.Introducer         = self.stream.byte(value=0x21)        # Extension Introducer (always 21h) 
        self.Label              = self.stream.byte(value=0xF9)        # Graphic Control Label (always F9h) 
        self.BlockSize          = self.stream.byte(value=0x04)        # Size of remaining fields (always 04h) 
        self.Packed             = self.stream.byte()                  # Method of graphics disposal to use 
        self.DelayTime          = self.stream.word()                  # Hundredths of seconds to wait	
        self.ColorIndex         = self.stream.byte()                  # Transparent Color Index 
        self.Terminator         = self.stream.byte(value=0x00)        # Block Terminator (always 0) 

        # computed
        self.unpack()  

    def write(self):
        self.internal_position=self.stream.pos
        self.pack()
        self.stream.write_byte(self.Introducer)
        self.stream.write_byte(self.Label)
        self.stream.write_byte(self.BlockSize)
        self.stream.write_byte(self.Packed)
        #print ("THE DEALY:{0}".format(self.DelayTime))
        self.stream.write_word(self.DelayTime)
        self.stream.write_byte(self.ColorIndex)
        self.stream.write_byte(self.Terminator)

    def pack(self):
        self.Packed=0
        self.Packed =self.Reserved             <<3
        self.Packed+=self.DisposalMethod       <<2
        self.Packed+=self.UserInputFlag        <<1
        self.Packed+=self.TransparentColorFlag

    def unpack(self):
        self.TransparentColorFlag =self.stream.bit(self.Packed,0)
        self.UserInputFlag        =self.stream.bit(self.Packed,1)
        self.DisposalMethod       =self.stream.bit(self.Packed,2,3)
        self.Reserved             =self.stream.bit(self.Packed,3)

    # is no color index is passed, nothing is transparent. 
    # if so flag is set to transparent available and the index is set
    def new(self,   DelayTime=1,
                    ColorIndex=None,
                    UserInputFlag=0,
                    DisposalMethod=0):

        self.Introducer= 0x21
        self.Label      =0xF9
        self.BlockSize  =0x04
        self.DelayTime  =DelayTime
        self.Terminator= 0x00

        if ColorIndex:
            TransparentColorFlag=1
            self.ColorIndex =ColorIndex
        else:
            TransparentColorFlag=0
            self.ColorIndex =0




        self.TransparentColorFlag =TransparentColorFlag
        self.UserInputFlag        =UserInputFlag
        self.DisposalMethod       =DisposalMethod
        self.Reserved             =0x00
        self.pack()


    def debug(self):
        print("Graphics Control Extension Block")
        print("  Offset:               {0:02X}".format(self.internal_position))
        print("  Introducer:           {0:02X}".format(self.Introducer))
        print("  Label:                {0:02X}".format(self.Label))
        print("  BlockSize:            {0}".format(self.BlockSize))
        print("  Packed:               {0}".format(self.Packed))
        print("  DelayTime:            {0}".format(self.DelayTime))
        print("  ColorIndex:           {0}".format(self.ColorIndex))
        print("  Terminator:           {0}".format(self.Terminator))
        print("  TransparentColorFlag: {0}".format(self.TransparentColorFlag))
        print("  UserInputFlag:        {0}".format(self.UserInputFlag))
        print("  DisposalMethod:       {0}".format(self.DisposalMethod))
        print("  Reserved:             {0}".format(self.Reserved))
    


# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2

import json 
import copy
import re

class asciicast_reader:
    def __init__(self,debug=None):
        self.debug=debug

    def load(self,filename):
        cast={}
        header={}
        events=[]
        with open(filename,'r') as content:
            header=json.loads(content.readline())
            for line in  content:
                events.append(json.loads(line))
        
        if 'title' not in header:
            header['title']=''
        

        return {'version':header['version'],
                'width':header['width'],
                'height':header['height'],
                'timestamp':header['timestamp'],
                'title':header['title'],
                'env':header['env'],
                'events':events
        }

    def write(self,file,stream):
        
        with open(file,"w") as outfile:
            header="{{'version':{0},'width':{1},'height':{2},'timestamp':{3},'title':{4},'env':{5}}}\n".format(stream['version'],stream['width'],stream['height'],stream['timestamp'],stream['title'],stream['env'])
            outfile.write(header)
            
            for event in stream['events']:
                o=copy.deepcopy(event[2])
                o = re.escape(o)

                outfile.write("[{0:.10f},'{1}',{2}]\n".format(event[0],event[1],repr(o)))



# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2


class CommentExtension:
    #  5 to 259 bytes in total length
    def __init__(self,stream):
        self.internal_position=stream.pos
        self.Introducer= stream.byte(value=0x21)        # Extension Introducer (always 21h)
        self.Label= stream.byte(value=0xFE)             # Comment Label (always FEh)
        self.block_size=stream.byte()
        data=[]
        while self.block_size>0:
            fragment=stream.byte(self.block_size)
            if self.block_size==1:
                data+=[fragment]
            else:
                data+=fragment
            self.block_size=stream.byte()
        self.CommentData=data
        #self.Terminator= stream.byte(value=0x00)       # Block Terminator (always 0)

    def debug(self):
        print("Comment Extension Block")
        print("  Offset: {0:02X}".format(self.internal_position))
        print("  Introducer: {0:02X}".format(self.Introducer))
        print("  Label: {0:02X}".format(self.Label))
        print("  CommentData: {0}".format(self.CommentData))
        #print("  Terminator: {0:02X}".format(self.Terminator))



# cython: profile=True

import json
# TODO pick transparent color index, none OR first pixel
# TODO disposal of last frame if end of loop
# TODO Loop based on # times or infinity

class canvas:

    def __init__(self):
        x=1

    def web(self,dest_file,gif_data):
        template="""<html>
    <head>
        <title>TTYGIF {0}, STDIO/TTY to GIF</title>
    </head>
    <body>
        <canvas id="ttygif" width="{1}" height="{2}" style="border:1px solid #000000;"></canvas>
        <script>
            var global_color_table={3}
            var frame_count={4}
            var frames={5}
            var header={6}
            var width={1}
            var height={2}
            var canvas = document.getElementById('ttygif');
            var ctx = canvas.getContext("2d");
            var id = ctx.createImageData(1,1);
            var id2 = ctx.createImageData({1},{2});
            var d  = id.data;                        
            var id2d  = id2.data;                        
            var mode=3;  // this changes the method used to blit the image.
                         // 1 is a rect per pixel. good for scaling i guess
                         // 2 is single image writes way slower
                         // 3 is single blit

            function put_pixel(x,y,color_index){{
                var color=global_color_table[color_index]
                if(mode==1){{
                    var r=color[0];
                    var g=color[1];
                    var b=color[2];
                    ctx.fillStyle = "rgb("+r+","+g+","+b+")";
                    ctx.fillRect( x, y, 1, 1 );
                }} 
                if(mode==2) {{
                    d[0]   = color[0];
                    d[1]   = color[1];
                    d[2]   = color[2];
                    d[3]   = 255;
                    ctx.putImageData( id, x, y ); 
                }}
                if(mode==3){{
                    var off = (y * id2.width + x) * 4;

                    id2d[off+0]   = color[0];
                    id2d[off+1]   = color[1];
                    id2d[off+2]   = color[2];
                    id2d[off+3]   = 255;

                }}
            }}

            function fill(color_index){{
                var color=global_color_table[color_index]
                var r=color[0];
                var g=color[1];
                var b=color[2];
            
                ctx.fillStyle = "rgb("+r+","+g+","+b+")";
                ctx.fillRect( 0, 0, width, height );
            }}

            function draw_frame(frame){{
                var cx=0;
                var cy=0;
                var color_index=0;
                x=frame['descriptor']['Left']
                y=frame['descriptor']['Top']
                width=frame['descriptor']['Width']
                height=frame['descriptor']['Height']
                image=frame['image']['data']
                len_of_frame=frame['image']['data'].length
                if(frame['gc'] && frame['gc']['TransparentColorFlag']==true){{
                    transparent=frame['gc']['ColorIndex']
                }} else {{
                    transparent=-1
                }}

                
                for(i=0;i<len_of_frame;i++){{
                    color_index=image[i]
                    if(color_index!=transparent){{
                        put_pixel(cx+x,cy+y,color_index)
                    }}

                    cx++;
                    if(cx==width){{
                        cx=0;
                        cy++;
                    }}
                }}
                if (mode==3){{
                    ctx.putImageData(id2,0,0);
            
                }}
            }}
            function sleep(milliseconds) {{
                var start = new Date().getTime();
                for (var i = 0; i < 1e7; i++) {{
                    
                    if ((new Date().getTime() - start) > milliseconds){{
                    break;
                    }}
                }}
            }}

            
            fill(header['BackgroundColor']);
            var cur_frame=0,last_frame=0;
            function rotate(){{
                
                if(frames[last_frame]['gc']){{
                    disposal=frames[last_frame]['gc']['DisposalMethod']
                    
                                        
                    if(disposal==1){{ //Do rewind to a disposal=0
                        for(i=last_frame-1;i>-1;i--){{
                            if(frames[i]['gc']){{
                                disposal=frames[i]['gc']['DisposalMethod']
                                if(disposal==0){{
                                    draw_frame(frames[i]);
                                    break;
                                }}
                    
                            }}
                        }}
                    }}
                    if (frames[cur_frame]['gc']){{
                        DELAY=frames[cur_frame]['gc']['DelayTime']*10;
                        last_frame=cur_frame;
                        if (DELAY!=0){{
                            setTimeout(rotate,DELAY);
                            cur_frame++;
                            if(cur_frame>=frames.length){{
                                cur_frame=0;
                            }}
                        }}
                        
                    }}

                }}
                draw_frame(frames[cur_frame])
                
            }}
            rotate();
                
        
        </script>

    </body>
</html>"""
        bg=gif_data['header'].BackgroundColor
        width=gif_data['header'].ScreenWidth
        height=gif_data['header'].ScreenHeight
        colors=self.render(gif_data['global_color_table'].colors)
        header=self.render(gif_data['header'])
        frames=self.render(gif_data['frames'])
        frame_count=json.dumps(gif_data['frame_count'],indent=4)
        doc=template.format( dest_file,     #0
                             width,         #1
                             height,        #2
                             colors,        #3
                             frame_count,   #4
                             frames,        #5
                             header
                             )
        file = open(dest_file,"w") 
        file.write(doc) 
        file.close() 
    
    def extract(self,data,output_file):
        doc=self.render(data)
        file = open(output_file,"w") 
        file.write(doc) 
        file.close() 
    
    def screen_canvas(self,screen,output_file):
        template="""<html>
    <head>
        <title>TTYGIF {0}, STDIO/TTY to GIF</title>
    </head>
    <body>
        <canvas id="ttygif" width="{1}" height="{2}" style="border:0px solid #000000;"></canvas>
        <script>
            var frame={3}
            var canvas = document.getElementById('ttygif');
            var ctx = canvas.getContext("2d");
            var id2 = ctx.createImageData({1},{2});
            var id2d  = id2.data;                        
          
            function put_pixel(x,y,color_index){{
                var color=frame['color_table'][color_index]
                var off = (y * id2.width + x) * 4;

                id2d[off+0]   = color[0];
                id2d[off+1]   = color[1];
                id2d[off+2]   = color[2];
                id2d[off+3]   = 255;
            }}

            function fill(color_index){{
                var color=frame['color_table'][color_index]
                var r=color[0];
                var g=color[1];
                var b=color[2];
            
                ctx.fillStyle = "rgb("+r+","+g+","+b+")";
                ctx.fillRect( 0, 0, frame['width'], frame['height'] );
            }}

            function draw_frame(){{
                var cx=0;
                var cy=0;
                var color_index=0;
                width=frame['width']
                height=frame['height']
                len_of_frame=frame['data'].length
                
                for(i=0;i<len_of_frame;i++){{
                    color_index=frame['data'][i]
                    put_pixel(cx,cy,color_index)
                    cx++;
                    if(cx==width){{
                        cx=0;
                        cy++;
                    }}
                }}
                ctx.putImageData(id2,0,0);
            }}

            
            fill(0);
            draw_frame()
            
        </script>

    </body>
</html>"""
        frame=self.render(screen)
        doc=template.format( output_file,     #0
                             screen.width,
                             screen.height,
                             frame,         #4
                             )
        file = open(output_file,"w") 
        file.write(doc) 
        file.close() 
    
    
    def render(self,obj,depth=0):
        """json like output for python objects, very loose"""
        unk_template='"???{0}???"'
        str_template='"{0}"'
        int_template="{0}"
        float_template="{0}"
        bool_template="{0}"
        array_template='['+'{0}'+']'
        tuple_template='"{0}":{1}'
        object_template='{{'+'{0}'+'}}'
        NULL="{}"
        fragment=""
        #if None == obj:
         #   return fragment

        if obj == None:
            fragment+=NULL

        elif isinstance(obj,str):
            fragment+=str_template.format(obj)

        elif isinstance(obj,bool):
            if obj==True:
                fragment+="true"    
            if obj==False:
                fragment+="false"    

        elif isinstance(obj,int):
            fragment+=int_template.format(obj)

        elif isinstance(obj,float):
            fragment+=float_template.format(obj)
        
        elif  isinstance(obj,list):
            partial=[]
            for item in obj:
                partial.append(self.render(item,depth=depth+1))
            if len(partial)>0:
                fragment+=array_template.format(",".join(map(str, partial)))
        elif isinstance(obj,dict):
            partial=[]
            #print (obj)
            for item in obj:
                partial.append(tuple_template.format(item,self.render(obj[item],depth=depth+1)))
            if len(partial)>0:
                fragment+=object_template.format(",".join(map(str, partial))) 
        elif isinstance(obj,object):
            partial=[]
            #print (obj)
            if hasattr(obj,'__dict__'):
                try:
                    for item in obj.__dict__:
                        render_res=self.render(obj.__dict__[item],depth=depth+1)
                        if not render_res or render_res=="":
                            continue
                        partial.append(tuple_template.format(item,render_res))
                    if len(partial)>0:
                        fragment+=object_template.format(",".join(map(str, partial))) 
                except:
                    pass
            else:
                try:
                    for item in obj:
                        render_res=self.render(obj[item],depth=depth+1)
                        if not render_res or render_res=="":
                            continue
                        partial.append(tuple_template.format(item,render_res))
                    if len(partial)>0:
                        fragment+=object_template.format(",".join(map(str, partial))) 
                except:
                    pass
        else:
            fragment+=unk_template.format("UNK",obj)
        return fragment


# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2

class PlainTextExtension:
    def __init__(self,stream):
        self.internal_position=stream.pos
        self.Introducer= stream.byte(value=0x21)         # Extension Introducer (always 21h) 
        self.Label= stream.byte(value=0x01)              # Extension Label (always 01h) 
        self.BlockSize= stream.byte(value=0x0C)          # Size of Extension Block (always 0Ch) 
        self.TextGridLeft= stream.word()                 # X position of text grid in pixels 
        self.TextGridTop= stream.word()                  # Y position of text grid in pixels 
        self.TextGridWidth= stream.word()                # Width of the text grid in pixels 
        self.TextGridHeight= stream.word()               # Height of the text grid in pixels 
        self.CellWidth= stream.byte()                    # Width of a grid cell in pixels 
        self.CellHeight= stream.byte()                   # Height of a grid cell in pixels 
        self.TextFgColorIndex= stream.byte()             # Text foreground color index value 
        self.TextBgColorIndex= stream.byte()             # Text background color index value 
        self.plainTextData= stream.byte(eod=0x00)        # The Plain Text data 

        self.SubBlockDataSize=0
        self.data_sub_blocks=[]
        SubBlockDataSize= stream.byte()
        while SubBlockDataSize!=0:
            self.SubBlockDataSize+=SubBlockDataSize
            self.data_sub_blocks+=stream.byte(SubBlockDataSize)      # Point to Application Data sub-blocks 
            SubBlockDataSize= stream.byte()
        

        self.Terminator= SubBlockDataSize #stream.byte(value=0x00)         # Block Terminator (always 0) 

        #self.Terminator= stream.byte(value=0x00)         # Block Terminator (always 0) 


    def debug(self):
        print("Plain Text Extension Block")
        print("  Offset: {0:02X}".format(self.internal_position))
        print("  Introducer: {0}".format(self.Introducer))
        print("  Label: {0}".format(self.Label))
        print("  BlockSize: {0}".format(self.BlockSize))
        print("  TextGridLeft: {0}".format(self.TextGridLeft))
        print("  TextGridTop: {0}".format(self.TextGridTop))
        print("  TextGridWidth: {0}".format(self.TextGridWidth))
        print("  TextGridHeight: {0}".format(self.TextGridHeight))
        print("  CellWidth: {0}".format(self.CellWidth))
        print("  CellHeight: {0}".format(self.CellHeight))
        print("  TextFgColorIndex: {0}".format(self.TextFgColorIndex))
        print("  TextBgColorIndex: {0}".format(self.TextBgColorIndex))
        print("  SubBlockDataSize: {0}".format(self.SubBlockDataSize))
        #print("  : {0}".format(self.plainTextData))
        print("  Terminator: {0}".format(self.Terminator))



# cython: profile=True

class application_extension:
    def __init__(self,stream):
        self.stream=stream



    def read(self):
        self.internal_position = self.stream.pos
        self.Introducer        = self.stream.byte(value=0x21)           # Extension Introducer (always 21h) 
        self.Label             = self.stream.byte(value=0xFF)           # Extension Label (always FFh) 
        self.BlockSize         = self.stream.byte(value=0x0B)           # Size of Extension Block (always 0Bh) 
        self.Identifier        = self.stream.string(8)                  # Application Identifier 
        self.AuthentCode       = self.stream.string(3)                  # Application Authentication Code 
        
        self.SubBlockDataSize=0
        self.data_sub_blocks=[]
        SubBlockDataSize= self.stream.byte()
        while SubBlockDataSize!=0:
            self.SubBlockDataSize+=SubBlockDataSize
            self.data_sub_blocks+=self.stream.byte(SubBlockDataSize)      # Point to Application Data sub-blocks 
            SubBlockDataSize= self.stream.byte()
        

        self.Terminator= SubBlockDataSize #stream.byte(value=0x00)         # Block Terminator (always 0) 


    def write(self):
        self.internal_position=self.stream.pos
        self.stream.write_byte(self.Introducer)
        self.stream.write_byte(self.Label)
        self.stream.write_byte(self.BlockSize)
        self.stream.write_string(self.Identifier,8)
        self.stream.write_string(self.AuthentCode,3)
        
        left=len(self.data_sub_blocks)
        index=0
        while left>0:
            if left>255:
                self.stream.write_byte(255)
                for b in self.data_sub_blocks[index:left]:
                    self.stream.write_byte(b)
                left-=255
            else:
                self.stream.write_byte(left)
                for b in self.data_sub_blocks[index:left]:
                    self.stream.write_byte(b)
                left=0

        
        self.stream.write_byte(self.Terminator)

    def new_netscape_block(self,loop_count=0xFFFF):
        if loop_count>0xFFFF:
            loop_count=0xFFFF
        self.Introducer        = 0x21
        self.Label             = 0xFF
        self.BlockSize         = 0x0B
        self.Identifier        = "NETSCAPE"
        self.AuthentCode       = "2.0"
        self.SubBlockDataSize  = 0x03
        self.data_sub_blocks   = [1,loop_count & 0xFF,(loop_count>>8) & 0xFF]
        self.Terminator        = 0x00
        #print("LOOP",loop_count & 0xFF,(loop_count>>8) & 0xFF)



    
    def debug(self):
        print("Application Extension Block")
        print("  Offset:           {0:02X}".format(self.internal_position))
        print("  Introducer:       {0}".format(self.Introducer))
        print("  Label:            {0}".format(self.Label))
        print("  BlockSize:        {0}".format(self.BlockSize))
        print("  Identifier:       {0}".format(self.Identifier))
        print("  AuthentCode:      {0}".format(self.AuthentCode))
        print("  SubBlockDataSize: {0:02X}".format(self.SubBlockDataSize))
        print("  Terminator:       {0:02X}".format(self.Terminator))
    

# 30D:   21 FF                  Application Extension block
#   0B           11         - eleven bytes of data follow
#   4E 45 54
#       53 43 41
#       50 45        NETSCAPE   - 8-character application name
#       32 2E 30     2.0        - application "authentication code"
#   03           3          - three more bytes of data
#   01           1          - data sub-block index (always 1)
#   FF FF        65535      - unsigned number of repetitions
#   00                      - end of App Extension block


# cython: language_level=2
import os
import sys
from setuptools import setup, find_packages
from setuptools.extension import Extension
from Cython.Build import cythonize


if '--build-cython' in sys.argv:
    # if this is is the system building the package from source, use the py files (or pyx)
    index = sys.argv.index('--build-cython')
    sys.argv.pop(index)  # Removes the '--foo'
    ext = '.py'
    ext2= '.pyx' # X when needed
    prefix=''
    print("Using Cython")
    USE_CYTHON=True
else:
    # if this is a package install, use the c files and build/register modules
    ext = '.c'
    ext2 = '.c'
    prefix=''
    USE_CYTHON=None    
# cython: linetrace=True
# cython: binding=True
# distutils: define_macros=CYTHON_TRACE_NOGIL=1

extensions = [
    #data import
    Extension("ttygif.asciicast.reader"                 ,[prefix+"./ttygif/asciicast/reader"               +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    #gif minipulation
    Extension("ttygif.gif.application_extension"        ,[prefix+"./ttygif/gif/application_extension"      +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.color_table"                  ,[prefix+"./ttygif/gif/color_table"                +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.CommentExtension"             ,[prefix+"./ttygif/gif/CommentExtension"           +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.graphics_control_extension"   ,[prefix+"./ttygif/gif/graphics_control_extension" +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.header"                       ,[prefix+"./ttygif/gif/header"                     +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.image_descriptor"             ,[prefix+"./ttygif/gif/image_descriptor"           +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.PlainTextExtension"           ,[prefix+"./ttygif/gif/PlainTextExtension"         +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.stream"                       ,[prefix+"./ttygif/gif/stream"                     +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.trailer"                      ,[prefix+"./ttygif/gif/trailer"                    +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tools.passthrough"                ,[prefix+"./ttygif/tools/passthrough"              +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.image"                        ,[prefix+"./ttygif/gif/image"                      +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.decode"                       ,[prefix+"./ttygif/gif/decode"                     +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.encode"                       ,[prefix+"./ttygif/gif/encode"                     +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.canvas"                       ,[prefix+"./ttygif/gif/canvas"                     +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.gif.gif"                          ,[prefix+"./ttygif/gif/gif"                        +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    # graphical rendering
    Extension("ttygif.tty.font"                         ,[prefix+"./ttygif/tty/font"                       +ext2], define_macros=[('CYTHON_TRACE', '1')]),
#    Extension("ttygif.tty.fonts"                        ,[prefix+"./ttygif/tty/fonts"                      +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tty.display_state"                ,[prefix+"./ttygif/tty/display_state"              +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tty.image"                        ,[prefix+"./ttygif/tty/image"                      +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tty.parser"                       ,[prefix+"./ttygif/tty/parser"                     +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tty.theme"                        ,[prefix+"./ttygif/tty/theme"                      +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tty.terminal_graphics"            ,[prefix+"./ttygif/tty/terminal_graphics"          +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tty.terminal_emulator"            ,[prefix+"./ttygif/tty/terminal_emulator"          +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    Extension("ttygif.tty.graphics"                     ,[prefix+"./ttygif/tty/graphics"                   +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    
    # gif orchestraton of frames and data
    Extension("ttygif.cast2gif"                         ,[prefix+"./ttygif/cast2gif"                       +ext2], define_macros=[('CYTHON_TRACE', '1')]),
    
    Extension("ttygif.version"                          ,[prefix+"./ttygif/version"                        +ext ], define_macros=[('CYTHON_TRACE', '1')]),
    #Extension("ttygif.cli"                              ,[prefix+"./ttygif/cli"                            +ext ], define_macros=[('CYTHON_TRACE', '1')]),
]
if USE_CYTHON:
    try:
        from Cython.Build import cythonize
        extensions = cythonize(extensions)
    except BaseException as be:
        print (be)
        print("You don't seem to have Cython installed.")
        print("www.cython.org")
        print("with pip ->pip install cython --user")
        print("with a pipenv ->pipenv install cython")
        print("Building")
        exit(1)
else:
    print("Not using CYTHON")


packages=[  'ttygif',
            'ttygif.tty',
            'ttygif.gif',
            'ttygif.tools',
            'ttygif.asciicast'
         ]
    
ver=sys.version_info
exec(open('ttygif/version.py').read())
setup(
    name='ttygif',
    version=__version__,
    packages=packages,
    include_package_data=True,
    url='https://github.com/chris17453/ttygif/',
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author='Charles Watkins',
    author_email='chris17453@gmail.com',
    description='A asciicast to gif utility',
    ext_modules=extensions,
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.7',
        "Development Status :: 3 - Alpha",
        "Operating System :: OS Independent",
    ],
    entry_points="""
        [console_scripts]
        ttygif = ttygif.cli:main
        """,
    compiler_directives={"language_level": ver.major},
    setup_requires=[
        # Setuptools 18.0 properly handles Cython extensions.
        'setuptools>=18.0',
        'cython',
        'future',
    ],    

)



import os
import struct 
import argparse


def file_2_compressed_array(input_file,output_file):
    with open(input_file, "rb") as file_r:
        fileContent = file_r.read()
    with open(output_file, "w") as file_w:
        file_w.write("""
# cython: linetrace=True
# **************************************
#   _     _       ___                
#  | |   (_)     |__ \               
#  | |__  _ _ __    ) |_ __  _   _   
#  | '_ \| | '_ \  / /| '_ \| | | |  
#  | |_) | | | | |/ /_| |_) | |_| |  
#  |_.__/|_|_| |_|____| .__/ \__, |  
#                     | |     __/ |  
#                     |_|    |___/   
#
# Auto generated by bin2py
# author: Charles Watkins
# web:    github.com/chris17453
# **************************************
from cpython cimport array
import array

file_name="{0}"
file_data=array.array('B',[
    """.format(input_file))
        content_len=len(fileContent)
        for i in range(0,content_len):
            if i>0:
                if i%16==0:
                    file_w.write("\n    ")
            chunk=struct.unpack('B', fileContent[i])[0] 

            file_w.write("0x{0:02X}".format(chunk))
            if i!=content_len-1:
                file_w.write(",")

        file_w.write("]\n")



if __name__=='__main__':

    parser = argparse.ArgumentParser(
        prog='bin2py',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description="""turn a binary file into a python array""")
    # actions

    parser.add_argument('-i', '--input',     help='binary file to load', default= None,metavar='FILE')
    parser.add_argument('-o', '--output',    help='python file to output', default= None,metavar='FILE')
    
    args = parser.parse_args()

    if args.input and args.output:
        file_2_compressed_array(args.input,args.output)
    else:
        parser.print_help()
    


# cython: profile=True
# cython: linetrace=True
# cython: binding=True
# cython: language_level=2


# Global header of gif file

class gif_header:
    def __init__(self,stream):
        self.stream=stream
        self.internal_position=self.stream.pos

    
    def read(self):
        self.internal_position=self.stream.pos
        gif_sig='GIF'
        gif_sig_ver=['87a','89a']
        # Header
        self.Signature               = self.stream.string(3,value=gif_sig)       # Header Signature (always "GIF") 
        self.Version                 = self.stream.string(3,value=gif_sig_ver)   # GIF format version("87a" or "89a")
        # Logical Screen Descriptor
        self.ScreenWidth             = self.stream.word()                        # Width of Display Screen in Pixels
        self.ScreenHeight            = self.stream.word()                        # Height of Display Screen in Pixels
        self.Packed                  = self.stream.byte()                        # Screen and Color Map Information
        self.BackgroundColor         = self.stream.byte()                        # Background Color Index
        self.AspectRatio             = self.stream.byte()                        # Pixel Aspect Ratio

        # computed
        self.unpack()

    def write(self):
        self.pack()
        self.stream.write_string(self.Signature,3)
        self.stream.write_string(self.Version,3)
        self.stream.write_word  (self.ScreenWidth)
        self.stream.write_word  (self.ScreenHeight)
        self.stream.write_byte  (self.Packed)
        self.stream.write_byte  (self.BackgroundColor)
        self.stream.write_byte  (self.AspectRatio)

    
    def new(self):
        self.Signature              = "GIF"
        self.Version                = "89a"
        self.ScreenWidth            = 320
        self.ScreenHeight           = 240
        self.Packed                 = 0
        self.BackgroundColor        = 0
        self.AspectRatio            = 0
        self.GlobalColorTableSize   = 0
        self.ColorTableSortFlag     = 0
        self.ColorResolution        = 1
        self.GlobalColorTableFlag   = 1
        self.GlobalColorTableLength = 0

        self.pack()

    def unpack(self):
        self.GlobalColorTableSize    = self.stream.bit(self.Packed,0,3)
        self.ColorTableSortFlag      = self.stream.bit(self.Packed,3)
        self.ColorResolution         = self.stream.bit(self.Packed,4,3)
        self.GlobalColorTableFlag    = self.stream.bit(self.Packed,7)
        self.GlobalColorTableLength  = 1 << (self.GlobalColorTableSize + 1)

    def pack(self):
        self.Packed= self.GlobalColorTableFlag <<7
        self.Packed+=self.ColorResolution      <<4
        self.Packed+=self.ColorTableSortFlag   <<3
        self.Packed+=self.GlobalColorTableSize
        self.GlobalColorTableLength  = 1 << (self.GlobalColorTableSize + 1)

        
        

    def debug(self):
        print("Header")
        print("  Offset:                 {0:02X}".format(self.internal_position))
        print("  Signature:              {0}".format(self.Signature))
        print("  Version:                {0}".format(self.Version))
        print("  ScreenWidth:            {0}".format(self.ScreenWidth))
        print("  ScreenHeight:           {0}".format(self.ScreenHeight))
        print("  Packed:                 {0}".format(self.Packed))
        print("  BackgroundColor:        {0}".format(self.BackgroundColor))
        print("  AspectRatio:            {0}".format(self.AspectRatio))
        print("  GlobalColorTableSize:   {0}".format(self.GlobalColorTableSize))
        print("  ColorTableSortFlag:     {0}".format(self.ColorTableSortFlag))
        print("  ColorResolution:        {0}".format(self.ColorResolution))
        print("  GlobalColorTableFlag:   {0}".format(self.GlobalColorTableFlag))
        print("  GlobalColorTableLength: {0}".format(self.GlobalColorTableLength))

